/***************************************************************************
 *   Nipper - The network infrastructure parser                            *
 *   Copyright Titania Limited 2009                                        *
 *   http://www.titania.co.uk                                              *
 *                                                                         *
 *   The software license should have been provided with this product in   *
 *   a file named LICENSE. All unlicensed use of Nipper is strictly        *
 *   prohibited.                                                           *
 *                                                                         *
 *   The license can be obtained online from the web site.                 *
 *                                                                         *
 ***************************************************************************/

#ifndef libnipper_H
#define libnipper_H

extern "C"
{

#include "config.h"
class Device;


/**
 * @short The main Nipper class.
 *
 * The Nipper class provides the main processing method for the processing
 * of device configurations. Use the class Config to configure Nipper prior to
 * processing and reporting.
 *
 * <b>Example:</b>\n
 *
 * \code
 *   #include <libnipper/libnipper.h>
 *   #include <libnipper/config.h>
 *   ...
 *   Nipper *nipper = 0;
 *   int errorCode = 0;
 *   ...
 *   // Create Nipper
 *   nipper = new (Nipper);
 *
 *   // Set the device configuration file for nipper to process...
 *   nipper->config->inputSource = "firewall.config";
 *
 *   // Process the configuration file...
 *   errorCode = nipper->process();
 *   if (errorCode == 0)
 *   {
 *
 *       // Generate the report from the configuration...
 *       errorCode = nipper->generateReport();
 *       if (errorCode == 0)
 *       {
 *
 *           // Output the report...
 *           errorCode = writeReport();
 *       }
 *   }
 *
 *   // If an error occured, output the error text...
 *   if (errorCode > 0)
 *       printf("Error: %s\n", nipper->getErrorText(errorCode));
 * \endcode
 */
class Nipper
{

  public:

	/**
	 * \brief Get the Nipper library version
	 *
	 * This method returns the Nipper library version.
	 *
	 * \return            The Nipper library version text.
	 */
	const char *getVersion();

	/**
	 * \brief Get the vulnerability database version details
	 *
	 * This method returns the Vulnerability database version.
	 *
	 * \return            The version text.
	 */
	const char *getVulnDBVersion();

	/**
	 * \brief The Nipper configuration settings
	 *
	 * Set the properties of this to configure Nipper.
	 */
	Config *config;

	/**
	 * \brief Process the device configuration file
	 *
	 * This method tells Nipper to process the configuration file based
	 * on the settings. Once the configuration has been processed a report
	 * can be generated.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 */
	int process();

	/**
	 * \brief Generate the report
	 *
	 * This method tells Nipper to generate a report based on the results of
	 * the configuration processing and the Nipper settings. It does not output
	 * the report.
	 * The report generation settings can be modified and generateReport run again
	 * without having to process the devices configuration again. This could be
	 * useful if you want to create seperate reports for configuration, security
	 * or any other report type.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 * \see Nipper::process
	 * \see Nipper::writeReport
	 */
	int generateReport();

	/**
	 * \brief Write the report to a file
	 *
	 * This method tells Nipper to write out a generated report. This method does not
	 * generate the report, the generateReport method should be called first.
	 *
	 * The report output settings can be modified and writeReport run again. This
	 * can enable the output to multiple report formats, without having to repeat
	 * any other processing.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 * \see Nipper::process
	 * \see Nipper::generateReport
	 */
	int writeReport();

	/**
	 * \brief Online update check
	 *
	 * This method checks with the update site for any Nipper updates. This method
	 * does not actually perform any updates. The numberOfUpdates method will return
	 * the number of updates available and the description of the update can be
	 * obtained using the updateDescription method. The update can be performed using
	 * the onlineUpdate method.
	 *
	 * Prior to any network operations, if you want to get feedback then set the
	 * networkCallback return function.
	 *
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 * \see Nipper::onlineUpdate
	 * \see Nipper::updateDescription
	 */
	int onlineUpdateCheck();

	/**
	 * \brief Number of updates
	 *
	 * This method returns the number of updates available. If an online update has
	 * not been performed, it will automatically be called.
	 *
	 * \see DeviceTypes
	 */
	int numberOfUpdates();

	/**
	 * \brief Online update
	 *
	 * This method performs an online update of all the enabled updates. Individual
	 * updates are all included by default, they can be removed using the
	 * disableUpdate method. Calling this without first calling onlineUpdateCheck
	 * will perform a full update.
	 *
	 * Prior to any network operations, if you want to get feedback then set the
	 * networkCallback return function.
	 *
	 * \param saveLocation A save location required by some updates.
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 * \see Nipper::onlineUpdate
	 */
	int onlineUpdate(const char *saveLocation = 0);

	/**
	 * \brief Get Update Title
	 *
	 * This method returns the title text of a specified update.
	 *
	 * \param update  The update number
	 * \return        It returns the update title text.
	 *
	 * \see Nipper::getErrorText
	 * \see Nipper::numberOfUpdates
	 * \see Nipper::onlineUpdate
	 */
	const char *getUpdateTitle(int update);

	/**
	 * \brief Get Update Version
	 *
	 * This method returns the version text of a specified update.
	 *
	 * \param update  The update number
	 * \return        It returns the update version text.
	 *
	 * \see Nipper::getErrorText
	 * \see Nipper::numberOfUpdates
	 * \see Nipper::onlineUpdate
	 */
	const char *getUpdateVersion(int update);

	/**
	 * \brief Get Update Description
	 *
	 * This method returns the description of a specified update.
	 *
	 * \param update  The update number
	 * \return        It returns the update description.
	 *
	 * \see Nipper::getErrorText
	 * \see Nipper::numberOfUpdates
	 * \see Nipper::onlineUpdate
	 */
	const char *getUpdateDescription(int update);

	/**
	 * \brief Get Update Save
	 *
	 * This method returns whether the update needs a save
	 * location being specified.
	 *
	 * \param update  The update number
	 * \return        It returns 1 if a save location needs
	 *                to be specified, else a 0 is returned.
	 *
	 * \see Nipper::getErrorText
	 * \see Nipper::numberOfUpdates
	 * \see Nipper::setUpdateSaveLocation
	 * \see Nipper::onlineUpdate
	 */
	int getUpdateSave(int update);

	/**
	 * \brief Activate Nipper
	 *
	 * This method is used to activate Nipper. Nipper MUST be activated before
	 * it can be used. The activation is performed online. Once activated it will
	 * only need re-activating if the licence expires.
	 *
	 * Prior to any network operations, if you want to get feedback then set the
	 * networkCallback return function.
	 *
	 * \param activationCode  This is the activation code that is obtained from
	 *                        the Titania web site.
	 * \param emailAddress    The email address related to the activation code.
	 *                        If they don't match the product will not be
	 *                        activated.
	 * \return      It returns 0 on success, otherwise it indicates an
	 *              error of some kind. The error text can be retreived
	 *              using the getErrorText method.
     * \see Nipper::getErrorText
	 */
	int activate(const char *activationCode, const char *emailAddress);
	int activate(const char *activationCode, const char *emailAddress, const char *returnedCode);
	const char *getSerial();

	/**
	 * \brief Is Nipper Activated?
	 *
	 * This method checks to see if Nipper is activated.
	 *
	 * \return     A 1 indicates that Nipper is activated, 0 is disabled.
	 */
	int isActivated();

	/**
	 * \brief Get licence holders name
	 *
	 * This method returns the licence holders name.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseeName();

	/**
	 * \brief Get licence holders address
	 *
	 * This method returns the licence holders address.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseeAddress();

	/**
	 * \brief Get licence holders phone number
	 *
	 * This method returns the licence holders phone number.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseePhone();

	/**
	 * \brief Get licence holders email address
	 *
	 * This method returns the licence holders email address.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseeEmail();

	/**
	 * \brief Get licence holders licence info
	 *
	 * This method returns the licence holders licence info.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseeLicence();

	/**
	 * \brief Get licence holders licence from date
	 *
	 * This method returns the licence holders licence from date.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseeFromDate();

	/**
	 * \brief Get licence holders licence to date
	 *
	 * This method returns the licence holders licence from date.
	 *
	 * \return      It returns the requested text.
	 */
	const char *getLicenseeToDate();

	/**
	 * \brief Set a proxy server
	 *
	 * This method sets an Internet proxy server configuration to
	 * use for Internet connections.
	 *
	 * \param server    The proxy server IP/hostname
	 * \param port      The proxy server TCP port
	 * \param username  The proxy server username
	 * \param password  The proxy server password (set to "********" to not
	 *                  change the existing password)
	 *
	 * \return      It returns 0 on success
	 */
	int setProxyServerDetails(const char *server, const char *port, const char *username = 0, const char *password = 0);

	/**
	 * \brief Get proxy server address
	 *
	 * This method returns the current proxy server IP address.
	 *
	 * \return      It returns the proxy server IP address/hostname
	 */
	const char *getProxyServer();

	/**
	 * \brief Get proxy server port
	 *
	 * This method returns the current proxy server port
	 *
	 * \return      It returns the proxy server port
	 */
	const char *getProxyServerPort();

	/**
	 * \brief Get proxy server username
	 *
	 * This method returns the current proxy server username. If
	 * anything is returned, set the password to "********".
	 *
	 * \return      It returns the proxy server username
	 */
	const char *getProxyServerUsername();

	/**
	 * \brief Set Network Callback
	 *
	 * This method sets the network callback function. This will
	 * be called during the network processing to give an indication
	 * of progress and report any issues.
	 *
	 * Details:
	 *     action = 0 - Nothing
	 *     action = 1 - Proxy Authentication Required (set using
	 *                  setProxyServerDetails)
	 *     action = 2 - Download progress (value = done, seconday =
	 *                  total)
	 *     action = 3 - Error (get error text using getErrorText(42))
	 *
	 * \return      It returns the proxy server username
	 */
	void setNetworkCallback(int (*networkCallback)(int action, int value, int secondary));

	/**
	 * \brief Get the error text
	 *
	 * This method gets the error text based on the code provided.
	 *
	 * \param errorCode   The error code returned by the process method.
	 * \return            The error text for the supplied code is returned.
	 * \see Nipper::process
	 */
	const char *getErrorText(int errorCode);

	/**
	 * \brief The Supported Nipper Device Configuration Type
	 *
	 * This enumeration is used to indicate what form the configuration
	 * is expected to take. For example, Cisco ASA configuration will be
	 * a file and a Nokia IP firewall configuration will be directory.
	 *
	 * \see DeviceTypes
	 */
	enum ConfigType
	{
		configFile = 0,
		configDir = 1,
		configEither = 2
	};

	/**
	 * \brief The Supported Nipper Device Types Structure
	 *
	 * This structure is used to create a list of devices that Nipper
	 * supports and can be used to specify what type of device the configuration
	 * is for.
	 *
	 * - deviceDescription (Contains an textual description of the device type)
	 * - commandLineOption (Contains the devices recommended cli option)
	 * - deviceId (Contains the ID that should be used to set the deviceType property in the Config class)
	 * - type (Identifies whether Nipper is expecting a configuration file or a configuration directory)
	 * - next (Points to the next entry in the list of supported devices, or 0 if it is the last entry)
	 *
	 * \see deviceTypes
	 * \see ConfigType
	 */
	struct SupportedDeviceTypes
	{
		const char *deviceDescription;
		const char *commandLineOption;
		Config::DeviceTypes deviceId;
		ConfigType type;
		struct SupportedDeviceTypes *next;
	};

	/**
	 * \brief The Supported Nipper Device Type List
	 *
	 * This is a Config::DeviceTypes list of devices supported by Nipper. A
	 * GUI/CLI can make use of this list to dynamically support whatever devices
	 * that Nipper supports.
	 *
	 * It is highly recommended that you use this structure to get a list of
	 * devices if you are not going to let Nipper do its auto-detection. Using
	 * this structure means that when new devices are added to Nipper your
	 * software will automatically support them. In the future, device support
	 * is likely to be added with the use of plugins.
	 *
	 * \see SupportedDeviceTypes
	 */
	SupportedDeviceTypes *deviceTypes;

	/**
	 * Constructor
	 */
	Nipper(const char *resourceDir = 0);

	/**
	 * Destructor
	 */
	~Nipper();

	/**
	 * \brief Set Update Frequency
	 *
	 * This method sets the online update frequency.
	 *
	 * 0 = daily
	 * 1 = weekly
	 * 2 = monthly
	 * 3 = never (not applicable to some licences)
	 */
	int setUpdateFrequency(unsigned int frequency);

	/**
	 * \brief Get Update Frequency
	 *
	 * This method gets the online update frequency.
	 *
	 * 0 = daily
	 * 1 = weekly
	 * 2 = monthly
	 * 3 = never (not applicable to some licences)
	 */
	unsigned int getUpdateFrequency();

	/**
	 * \brief Nipper functionality level
	 *
	 * This is highlights to a client application what level of access is
	 * granted by Nipper. A value of 2 = Home User and 4 = Evaluation.
	 *
	 * Modifying this value will NOT change the access level, it is used
	 * by Nipper to inform about the level of access.
	 */
	unsigned int functionalityLevel;

	/**
	 * \brief Nipper reporting level
	 *
	 * This is highlights to a client application what level of access is
	 * granted by Nipper. A value of 2 = Home User and 4 = Evaluation.
	 *
	 * Modifying this value will NOT change the access level, it is used
	 * by Nipper to inform about the level of access.
	 */
	unsigned int reportingLevel;

  private:
	const char *resourceDirectory;
	Device *device;
};

}

#endif
